@@ -723,17 +723,17 @@ function WebGLWorker() {
       source = source.replace(/\n/g, '|\n'); // barrier between lines, to make regexing easier
       var newItems = source.match(new RegExp(type + '\\s+\\w+\\s+[\\w,\\s\[\\]]+;', 'g'));
       if (!newItems) return;
-      newItems.forEach(function(item) {
+      newItems.forEach((item) => {
         var m = new RegExp(type + '\\s+(\\w+)\\s+([\\w,\\s\[\\]]+);').exec(item);
         assert(m);
-        m[2].split(',').map(function(name) { name = name.trim(); return name.search(/\s/) >= 0 ? '' : name }).filter(function(name) { return !!name }).forEach(function(name) {
+        m[2].split(',').map((name) => { name = name.trim(); return name.search(/\s/) >= 0 ? '' : name }).filter((name) => !!name).forEach((name) => {
           var size = 1;
           var open = name.indexOf('[');
           var fullname = name;
           if (open >= 0) {
             var close = name.indexOf(']');
-            size = parseInt(name.substring(open+1, close));
-            name = name.substr(0, open);
+            size = parseInt(name.slice(open+1, close));
+            name = name.slice(0, open);
             fullname = name + '[0]';
           }
           if (!obj[name]) {
@@ -754,7 +754,7 @@ function WebGLWorker() {
 
     var existingAttributes = {};
 
-    program.shaders.forEach(function(shader) {
+    program.shaders.forEach((shader) => {
       parseElementType(shader, 'uniform', program.uniforms, program.uniformVec);
       parseElementType(shader, 'attribute', existingAttributes, null);
     });
@@ -801,8 +801,8 @@ function WebGLWorker() {
     var open = name.indexOf('[');
     if (open >= 0) {
       var close = name.indexOf(']');
-      index = parseInt(name.substring(open+1, close));
-      name = name.substr(0, open);
+      index = parseInt(name.slice(open+1, close));
+      name = name.slice(0, open);
     }
     if (!(name in program.uniforms)) return null;
     var id = nextId++;
@@ -1153,8 +1153,8 @@ function WebGLWorker() {
   Browser.doSwapBuffers = postRAF;
 
   var trueRAF = window.requestAnimationFrame;
-  window.requestAnimationFrame = function(func) {
-    trueRAF(function() {
+  window.requestAnimationFrame = (func) => {
+    trueRAF(() => {
       if (preRAF() === false) {
         window.requestAnimationFrame(func); // skip this frame, do it later
         return;
