@@ -24,6 +24,7 @@
 #include <string>
 
 #define EMSCRIPTEN_ALWAYS_INLINE __attribute__((always_inline))
+#define EMBIND_VISIBILITY_DEFAULT __attribute__((visibility("default")))
 
 #ifndef EMSCRIPTEN_HAS_UNBOUND_TYPE_NAMES
 #define EMSCRIPTEN_HAS_UNBOUND_TYPE_NAMES 1
@@ -45,8 +46,12 @@ typedef const void* TYPEID;
 // just need a unique identifier per type and polymorphic type
 // identification.
 
+template<typename T>
+static inline constexpr bool IsCanonicalized = std::is_same<T, typename std::decay<T>::type>::value;
+
 template<typename T>
 struct CanonicalizedID {
+    static_assert(IsCanonicalized<T>, "T should not be a reference or cv-qualified");
     static char c;
     static constexpr TYPEID get() {
         return &c;
@@ -64,6 +69,7 @@ struct Canonicalized {
 template<typename T>
 struct LightTypeID {
     static constexpr TYPEID get() {
+        static_assert(IsCanonicalized<T>, "T should not be a reference or cv-qualified");
         if (has_unbound_type_names) {
 #if __has_feature(cxx_rtti)
             return &typeid(T);
@@ -82,6 +88,7 @@ struct LightTypeID {
 
 template<typename T>
 constexpr TYPEID getLightTypeID(const T& value) {
+    static_assert(IsCanonicalized<T>, "T should not be a reference or cv-qualified");
     if (has_unbound_type_names) {
 #if __has_feature(cxx_rtti)
         return &typeid(value);
@@ -106,6 +113,7 @@ struct TypeID {
 
 template<typename T>
 struct TypeID<std::unique_ptr<T>> {
+    static_assert(std::is_class<T>::value, "The type for a std::unique_ptr binding must be a class.");
     static constexpr TYPEID get() {
         return TypeID<T>::get();
     }
@@ -116,6 +124,14 @@ struct TypeID<T*> {
     static_assert(!std::is_pointer<T*>::value, "Implicitly binding raw pointers is illegal.  Specify allow_raw_pointer<arg<?>>");
 };
 
+namespace rvp {
+
+struct default_tag {};
+struct take_ownership : public default_tag {};
+struct reference : public default_tag {};
+
+} // end namespace rvp
+
 template<typename T>
 struct AllowedRawPointer {
 };
@@ -127,6 +143,18 @@ struct TypeID<AllowedRawPointer<T>> {
     }
 };
 
+template<typename T>
+struct TypeID<const T> : TypeID<T> {
+};
+
+template<typename T>
+struct TypeID<T&> : TypeID<T> {
+};
+
+template<typename T>
+struct TypeID<T&&> : TypeID<T> {
+};
+
 // ExecutePolicies<>
 
 template<typename... Policies>
@@ -244,16 +272,16 @@ struct WithPolicies {
 template<typename T, typename = void>
 struct BindingType;
 
-#define EMSCRIPTEN_DEFINE_NATIVE_BINDING_TYPE(type)                 \
-template<>                                                  \
-struct BindingType<type> {                                  \
-    typedef type WireType;                                  \
-    constexpr static WireType toWireType(const type& v) {   \
-        return v;                                           \
-    }                                                       \
-    constexpr static type fromWireType(WireType v) {        \
-        return v;                                           \
-    }                                                       \
+#define EMSCRIPTEN_DEFINE_NATIVE_BINDING_TYPE(type)                            \
+template<>                                                                     \
+struct BindingType<type> {                                                     \
+    typedef type WireType;                                                     \
+    constexpr static WireType toWireType(const type& v, rvp::default_tag) {    \
+        return v;                                                              \
+    }                                                                          \
+    constexpr static type fromWireType(WireType v) {                           \
+        return v;                                                              \
+    }                                                                          \
 }
 
 EMSCRIPTEN_DEFINE_NATIVE_BINDING_TYPE(char);
@@ -272,13 +300,21 @@ EMSCRIPTEN_DEFINE_NATIVE_BINDING_TYPE(uint64_t);
 
 template<>
 struct BindingType<void> {
-    typedef void WireType;
+    // Using empty struct instead of void is ABI-compatible, but makes it easier
+    // to work with wire types in a generic template context, as void can't be
+    // stored in local variables or passed around but empty struct can.
+    // TODO: switch to std::monostate when we require C++17.
+    struct WireType {};
+
+    static void fromWireType(WireType) {
+        // No-op, as void has no value.
+    }
 };
 
 template<>
 struct BindingType<bool> {
     typedef bool WireType;
-    static WireType toWireType(bool b) {
+    static WireType toWireType(bool b, rvp::default_tag) {
         return b;
     }
     static bool fromWireType(WireType wt) {
@@ -294,7 +330,7 @@ struct BindingType<std::basic_string<T>> {
         size_t length;
         T data[1]; // trailing data
     }* WireType;
-    static WireType toWireType(const String& v) {
+    static WireType toWireType(const String& v, rvp::default_tag) {
         WireType wt = (WireType)malloc(sizeof(size_t) + v.length() * sizeof(T));
         wt->length = v.length();
         memcpy(wt->data, v.data(), v.length() * sizeof(T));
@@ -313,16 +349,9 @@ template<typename T>
 struct BindingType<T&> : public BindingType<T> {
 };
 
-template<typename T>
-struct BindingType<const T&> : public BindingType<T> {
-};
-
 template<typename T>
 struct BindingType<T&&> {
     typedef typename BindingType<T>::WireType WireType;
-    static WireType toWireType(const T& v) {
-        return BindingType<T>::toWireType(v);
-    }
     static T fromWireType(WireType wt) {
         return BindingType<T>::fromWireType(wt);
     }
@@ -331,9 +360,19 @@ struct BindingType<T&&> {
 template<typename T>
 struct BindingType<T*> {
     typedef T* WireType;
-    static WireType toWireType(T* p) {
+
+    static WireType toWireType(T* p, rvp::default_tag) {
         return p;
     }
+
+    static WireType toWireType(T* p, rvp::take_ownership) {
+        return p;
+    }
+
+    static WireType toWireType(T* p, rvp::reference) {
+        return p;
+    }
+
     static T* fromWireType(WireType wt) {
         return wt;
     }
@@ -344,12 +383,19 @@ struct GenericBindingType {
     typedef typename std::remove_reference<T>::type ActualT;
     typedef ActualT* WireType;
 
-    static WireType toWireType(const T& v) {
-        return new T(v);
+    template<typename R>
+    static WireType toWireType(R&& v, rvp::default_tag) {
+        return new ActualT(v);
     }
 
-    static WireType toWireType(T&& v) {
-        return new T(std::forward<T>(v));
+    template<typename R>
+    static WireType toWireType(R&& v, rvp::take_ownership) {
+        return new ActualT(std::move(v));
+    }
+
+    template<typename R>
+    static WireType toWireType(R&& v, rvp::reference) {
+        return &v;
     }
 
     static ActualT& fromWireType(WireType p) {
@@ -361,8 +407,8 @@ template<typename T>
 struct GenericBindingType<std::unique_ptr<T>> {
     typedef typename BindingType<T*>::WireType WireType;
 
-    static WireType toWireType(std::unique_ptr<T> p) {
-        return BindingType<T*>::toWireType(p.release());
+    static WireType toWireType(std::unique_ptr<T> p, rvp::default_tag) {
+        return BindingType<T*>::toWireType(p.release(), rvp::default_tag{});
     }
 
     static std::unique_ptr<T> fromWireType(WireType wt) {
@@ -374,7 +420,7 @@ template<typename Enum>
 struct EnumBindingType {
     typedef Enum WireType;
 
-    static WireType toWireType(Enum v) {
+    static WireType toWireType(Enum v, rvp::default_tag) {
         return v;
     }
     static Enum fromWireType(WireType v) {
@@ -407,7 +453,7 @@ constexpr bool typeSupportsMemoryView() {
 } // namespace internal
 
 template<typename ElementType>
-struct memory_view {
+struct EMBIND_VISIBILITY_DEFAULT memory_view {
     memory_view() = delete;
     explicit memory_view(size_t size, const ElementType* data)
         : size(size)
@@ -441,7 +487,7 @@ struct BindingType<memory_view<ElementType>> {
     // on the C++ side, nor is toWireType implemented in
     // JavaScript.)
     typedef memory_view<ElementType> WireType;
-    static WireType toWireType(const memory_view<ElementType>& mv) {
+    static WireType toWireType(const memory_view<ElementType>& mv, rvp::default_tag) {
         return mv;
     }
 };
