@@ -2,9 +2,8 @@
 // Emscripten is available under two separate licenses, the MIT license and the
 // University of Illinois/NCSA Open Source License.  Both these licenses can be
 // found in the LICENSE file.
-// This file defines the global state of the new file system.
-// Current Status: Work in Progress.
-// See https://github.com/emscripten-core/emscripten/issues/15041.
+
+// This file defines the global state.
 
 #include <emscripten/threading.h>
 
@@ -56,8 +55,7 @@ WasmFS::WasmFS() : rootDirectory(initRootDirectory()), cwd(rootDirectory) {
 // (Note that this means we can't find leaks inside WasmFS code itself, but that
 // seems fundamentally impossible for the above reasons, unless we made LSan log
 // its findings in a way that does not depend on normal file I/O.)
-__attribute__((weak)) extern "C" void __lsan_do_leak_check(void) {
-}
+__attribute__((weak)) extern "C" void __lsan_do_leak_check(void) {}
 
 extern "C" void wasmfs_flush(void) {
   // Flush musl libc streams.
@@ -161,7 +159,7 @@ void WasmFS::preloadFiles() {
     std::shared_ptr<Directory> parentDir;
     if (parsed.getError() ||
         !(parentDir = parsed.getFile()->dynCast<Directory>())) {
-      emscripten_console_error(
+      emscripten_err(
         "Fatal error during directory creation in file preloading.");
       abort();
     }
@@ -188,7 +186,7 @@ void WasmFS::preloadFiles() {
 
     auto parsed = path::parseParent(fileName);
     if (parsed.getError()) {
-      emscripten_console_error("Fatal error during file preloading");
+      emscripten_err("Fatal error during file preloading");
       abort();
     }
     auto& [parent, childName] = parsed.getParentChild();
