@@ -29,11 +29,12 @@ import platform
 import random
 import sys
 import unittest
+import time
 
 # Setup
 
 __rootpath__ = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-sys.path.append(__rootpath__)
+sys.path.insert(0, __rootpath__)
 
 import jsrun
 import parallel_testsuite
@@ -71,6 +72,8 @@ passing_core_test_modes = [
   'wasm64',
   'wasm64_v8',
   'wasm64_4gb',
+  'esm_integration',
+  'instance',
 ]
 
 # The default core test mode, used when none is specified
@@ -106,7 +109,7 @@ def check_js_engines():
   working_engines = [e for e in config.JS_ENGINES if jsrun.check_engine(e)]
   if len(working_engines) < len(config.JS_ENGINES):
     print('Not all the JS engines in JS_ENGINES appears to work.')
-    exit(1)
+    sys.exit(1)
 
   if common.EMTEST_ALL_ENGINES:
     print('(using ALL js engines)')
@@ -114,7 +117,7 @@ def check_js_engines():
 
 def get_and_import_modules():
   modules = []
-  for filename in glob.glob(os.path.join(os.path.dirname(__file__), 'test*.py')):
+  for filename in glob.glob(os.path.join(common.TEST_ROOT, 'test*.py')):
     module_dir, module_file = os.path.split(filename)
     module_name, module_ext = os.path.splitext(module_file)
     __import__(module_name)
@@ -135,7 +138,7 @@ def get_all_tests(modules):
 
 
 def get_crossplatform_tests(modules):
-  suites = ['core0', 'other'] # We don't need all versions of every test
+  suites = ['core0', 'other', 'sanity'] # We don't need all versions of every test
   crossplatform_tests = []
   # Walk over the test suites and find the test functions with the
   # is_crossplatform_test attribute applied by @crossplatform decorator
@@ -285,7 +288,7 @@ def load_test_suites(args, modules, start_at, repeat):
         unmatched_test_names.remove(name)
       except AttributeError:
         pass
-    if len(names_in_module):
+    if names_in_module:
       loaded_tests = loader.loadTestsFromNames(sorted(names_in_module), m)
       tests = flattened_tests(loaded_tests)
       suite = suite_for_module(m, tests)
@@ -303,15 +306,15 @@ def load_test_suites(args, modules, start_at, repeat):
   if not found_start:
     utils.exit_with_error(f'unable to find --start-at test: {start_at}')
   if total_tests == 1 or parallel_testsuite.num_cores() == 1:
-    common.EMTEST_SAVE_DIR = True
+    # TODO: perhaps leave it at 2 if it was 2 before?
+    common.EMTEST_SAVE_DIR = 1
   return suites, unmatched_test_names
 
 
 def flattened_tests(loaded_tests):
   tests = []
   for subsuite in loaded_tests:
-    for test in subsuite:
-      tests.append(test)
+    tests.extend(subsuite)
   return tests
 
 
@@ -329,8 +332,8 @@ def run_tests(options, suites):
   resultMessages = []
   num_failures = 0
 
-  print('Test suites:')
-  print([s[0] for s in suites])
+  if len(suites) > 1:
+    print('Test suites:', [s[0] for s in suites])
   # Run the discovered tests
 
   # We currently don't support xmlrunner on macOS M1 runner since
@@ -346,6 +349,8 @@ def run_tests(options, suites):
   else:
     testRunner = unittest.TextTestRunner(verbosity=2, failfast=options.failfast)
 
+  total_core_time = 0
+  run_start_time = time.perf_counter()
   for mod_name, suite in suites:
     print('Running %s: (%s tests)' % (mod_name, suite.countTestCases()))
     res = testRunner.run(suite)
@@ -353,6 +358,11 @@ def run_tests(options, suites):
            (mod_name, res.testsRun, len(res.errors), len(res.failures), len(res.skipped)))
     num_failures += len(res.errors) + len(res.failures) + len(res.unexpectedSuccesses)
     resultMessages.append(msg)
+    if hasattr(res, 'core_time'):
+      total_core_time += res.core_time
+  total_run_time = time.perf_counter() - run_start_time
+  if total_core_time > 0:
+    print('Total core time: %.3fs. Wallclock time: %.3fs. Parallelization: %.2fx.' % (total_core_time, total_run_time, total_core_time / total_run_time))
 
   if len(resultMessages) > 1:
     print('====================')
@@ -364,7 +374,7 @@ def run_tests(options, suites):
   return num_failures
 
 
-def parse_args(args):
+def parse_args():
   parser = argparse.ArgumentParser(prog='runner.py', description=__doc__)
   parser.add_argument('--save-dir', action='store_true',
                       help='Save the temporary directory used during for each '
@@ -398,7 +408,6 @@ def parse_args(args):
 def configure():
   common.EMTEST_BROWSER = os.getenv('EMTEST_BROWSER')
   common.EMTEST_DETECT_TEMPFILE_LEAKS = int(os.getenv('EMTEST_DETECT_TEMPFILE_LEAKS', '0'))
-  common.EMTEST_SAVE_DIR = int(os.getenv('EMTEST_SAVE_DIR', '0'))
   common.EMTEST_ALL_ENGINES = int(os.getenv('EMTEST_ALL_ENGINES', '0'))
   common.EMTEST_SKIP_SLOW = int(os.getenv('EMTEST_SKIP_SLOW', '0'))
   common.EMTEST_SKIP_FLAKY = int(os.getenv('EMTEST_SKIP_FLAKY', '0'))
@@ -406,7 +415,6 @@ def configure():
   common.EMTEST_LACKS_NATIVE_CLANG = int(os.getenv('EMTEST_LACKS_NATIVE_CLANG', '0'))
   common.EMTEST_REBASELINE = int(os.getenv('EMTEST_REBASELINE', '0'))
   common.EMTEST_VERBOSE = int(os.getenv('EMTEST_VERBOSE', '0')) or shared.DEBUG
-  common.EMTEST_FORCE64 = int(os.getenv('EMTEST_FORCE64', '0'))
   if common.EMTEST_VERBOSE:
     logging.root.setLevel(logging.DEBUG)
 
@@ -414,13 +422,14 @@ def configure():
   parallel_testsuite.NUM_CORES = os.environ.get('EMTEST_CORES') or os.environ.get('EMCC_CORES')
 
 
-def main(args):
-  options = parse_args(args)
+def main():
+  options = parse_args()
 
   # Some options make sense being set in the environment, others not-so-much.
   # TODO(sbc): eventually just make these command-line only.
   if os.getenv('EMTEST_SAVE_DIR'):
-    print('Prefer --save-dir over setting $EMTEST_SAVE_DIR')
+    print('ERROR: use --save-dir instead of EMTEST_SAVE_DIR=1, and --no-clean instead of EMTEST_SAVE_DIR=2')
+    return 1
   if os.getenv('EMTEST_REBASELINE'):
     print('Prefer --rebaseline over setting $EMTEST_REBASELINE')
   if os.getenv('EMTEST_VERBOSE'):
@@ -443,10 +452,10 @@ def main(args):
 
   set_env('EMTEST_BROWSER', options.browser)
   set_env('EMTEST_DETECT_TEMPFILE_LEAKS', options.detect_leaks)
+  if options.save_dir:
+    common.EMTEST_SAVE_DIR = 1
   if options.no_clean:
-    set_env('EMTEST_SAVE_DIR', 2)
-  else:
-    set_env('EMTEST_SAVE_DIR', options.save_dir)
+    common.EMTEST_SAVE_DIR = 2
   set_env('EMTEST_SKIP_SLOW', options.skip_slow)
   set_env('EMTEST_ALL_ENGINES', options.all_engines)
   set_env('EMTEST_REBASELINE', options.rebaseline)
@@ -495,7 +504,7 @@ configure()
 
 if __name__ == '__main__':
   try:
-    sys.exit(main(sys.argv))
+    sys.exit(main())
   except KeyboardInterrupt:
     logger.warning('KeyboardInterrupt')
     sys.exit(1)
