@@ -4,6 +4,7 @@
 # found in the LICENSE file.
 
 import os
+import shutil
 
 TAG = 'version_1'
 HASH = '929e8d6003c06ae09593021b83323c8f1f54532b67b8ba189f4aedce52c25dc182bac474de5392c46ad5b0dea5a24928e4ede1492d52f4dd5cd58eea9be4dba7'
@@ -17,9 +18,11 @@ def get(ports, settings, shared):
   ports.fetch_project('ogg', f'https://github.com/emscripten-ports/ogg/archive/{TAG}.zip', sha512hash=HASH)
 
   def create(final):
-    source_path = os.path.join(ports.get_dir(), 'ogg', 'Ogg-' + TAG)
-    ports.write_file(os.path.join(source_path, 'include', 'ogg', 'config_types.h'), config_types_h)
-    ports.install_header_dir(os.path.join(source_path, 'include', 'ogg'), 'ogg')
+    source_path = ports.get_dir('ogg', 'Ogg-' + TAG)
+    config_types_h = os.path.join(os.path.dirname(__file__), 'ogg/config_types.h')
+    shutil.copyfile(config_types_h, os.path.join(source_path, 'include/ogg/config_types.h'))
+    ports.install_headers(os.path.join(source_path, 'include', 'ogg'), target='ogg')
+    ports.make_pkg_config('ogg', TAG, '-sUSE_OGG')
     ports.build_port(os.path.join(source_path, 'src'), final, 'ogg')
 
   return [shared.cache.get_lib('libogg.a', create)]
@@ -29,38 +32,5 @@ def clear(ports, settings, shared):
   shared.cache.erase_lib('libogg.a')
 
 
-def process_args(ports):
-  return []
-
-
 def show():
-  return 'ogg (USE_OGG=1; zlib license)'
-
-
-config_types_h = '''\
-#ifndef __CONFIG_TYPES_H__
-#define __CONFIG_TYPES_H__
-
-/* these are filled in by configure */
-#define INCLUDE_INTTYPES_H 1
-#define INCLUDE_STDINT_H 1
-#define INCLUDE_SYS_TYPES_H 1
-
-#if INCLUDE_INTTYPES_H
-#  include <inttypes.h>
-#endif
-#if INCLUDE_STDINT_H
-#  include <stdint.h>
-#endif
-#if INCLUDE_SYS_TYPES_H
-#  include <sys/types.h>
-#endif
-
-typedef int16_t ogg_int16_t;
-typedef uint16_t ogg_uint16_t;
-typedef int32_t ogg_int32_t;
-typedef uint32_t ogg_uint32_t;
-typedef int64_t ogg_int64_t;
-
-#endif
-'''
+  return 'ogg (-sUSE_OGG or --use-port=ogg; zlib license)'
