@@ -12,28 +12,48 @@ import tempfile
 import time
 from contextlib import ContextDecorator
 
-sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 logger = logging.getLogger('profiler')
 
-from tools import response_file
+from . import response_file
 
 EMPROFILE = int(os.getenv('EMPROFILE', '0'))
 
 
 class Logger(ContextDecorator):
+  depth = 0
+
   def __init__(self, name):
     self.name = name
 
+  def __call__(self, func):
+    if self.name is None:
+      self.name = func.__name__
+    return super().__call__(func)
+
   def __enter__(self):
+    if EMPROFILE == 2:
+      indentation = '  ' * Logger.depth
+      logger.info('%sstart block "%s"', indentation, self.name)
+      Logger.depth += 1
     self.start = time.time()
 
-  def __exit__(self, type, value, traceback):
+  def __exit__(self, exc_type, value, traceback):
     # When a block ends debug log the total duration.
     now = time.time()
-    logger.debug('block "%s" took %.2f seconds', self.name, now - self.start)
+    duration = now - self.start
+    if exc_type:
+      msg = 'block "%s" raised an exception after %.3f seconds'
+    else:
+      msg = 'block "%s" took %.3f seconds'
+    if EMPROFILE == 2:
+      Logger.depth -= 1
+      indentation = '  ' * Logger.depth
+      logger.info(indentation + msg, self.name, duration)
+    else:
+      logger.debug(msg, self.name, duration)
 
 
-if EMPROFILE:
+if EMPROFILE == 1:
   original_sys_exit = sys.exit
   original_subprocess_call = subprocess.call
   original_subprocess_check_call = subprocess.check_call
@@ -132,7 +152,7 @@ if EMPROFILE:
 
     @staticmethod
     def escape_args(args):
-      return map(lambda arg: ToolchainProfiler.escape_string(arg), args)
+      return [ToolchainProfiler.escape_string(arg) for arg in args]
 
     @staticmethod
     def record_process_start(write_log_entry=True):
@@ -168,12 +188,7 @@ if EMPROFILE:
 
     @staticmethod
     def record_subprocess_spawn(process_pid, process_cmdline):
-      expanded_cmdline = []
-      for item in process_cmdline:
-        if item.startswith('@'):
-          expanded_cmdline += response_file.read_response_file(item)
-        else:
-          expanded_cmdline.append(item)
+      expanded_cmdline = response_file.substitute_response_files(process_cmdline)
 
       with ToolchainProfiler.log_access() as f:
         f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(ToolchainProfiler.escape_args(expanded_cmdline)) + '"]}')
@@ -217,18 +232,22 @@ if EMPROFILE:
     class ProfileBlock(Logger):
       def __init__(self, block_name):
         super().__init__(block_name)
-        self.block_name = block_name
+        self.name = block_name
 
       def __enter__(self):
-        ToolchainProfiler.enter_block(self.block_name)
+        ToolchainProfiler.enter_block(self.name)
 
       def __exit__(self, type, value, traceback):
-        ToolchainProfiler.exit_block(self.block_name)
+        ToolchainProfiler.exit_block(self.name)
 
     @staticmethod
     def profile_block(block_name):
       return ToolchainProfiler.ProfileBlock(ToolchainProfiler.escape_string(block_name))
 
+    @staticmethod
+    def profile():
+      return ToolchainProfiler.ProfileBlock(None)
+
     @staticmethod
     def imaginary_pid():
       ToolchainProfiler.imaginary_pid_ -= 1
@@ -248,3 +267,7 @@ else:
     @staticmethod
     def profile_block(block_name):
       return Logger(block_name)
+
+    @staticmethod
+    def profile():
+      return Logger(None)
