@@ -3,10 +3,10 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-import contextlib
 import os
 import shutil
 import sys
+from functools import wraps
 from pathlib import Path
 
 from . import diagnostics
@@ -47,15 +47,13 @@ def removeprefix(string, prefix):
   return string
 
 
-@contextlib.contextmanager
-def chdir(dir):
-  """A context manager that performs actions in the given directory."""
-  orig_cwd = os.getcwd()
-  os.chdir(dir)
-  try:
-    yield
-  finally:
-    os.chdir(orig_cwd)
+def convert_line_endings_in_file(filename, to_eol):
+  if to_eol == os.linesep:
+    assert os.path.exists(filename)
+    return # No conversion needed
+
+  text = read_file(filename)
+  write_file(filename, text, line_endings=to_eol)
 
 
 def read_file(file_path):
@@ -70,10 +68,14 @@ def read_binary(file_path):
     return fh.read()
 
 
-def write_file(file_path, text):
+def write_file(file_path, text, line_endings=None):
   """Write to a file opened in text mode"""
-  with open(file_path, 'w', encoding='utf-8') as fh:
-    fh.write(text)
+  if line_endings and line_endings != os.linesep:
+    text = text.replace('\n', line_endings)
+    write_binary(file_path, text.encode('utf-8'))
+  else:
+    with open(file_path, 'w', encoding='utf-8') as fh:
+      fh.write(text)
 
 
 def write_binary(file_path, contents):
@@ -84,9 +86,8 @@ def write_binary(file_path, contents):
 
 def delete_file(filename):
   """Delete a file (if it exists)."""
-  if not os.path.exists(filename):
-    return
-  os.remove(filename)
+  if os.path.lexists(filename):
+    os.remove(filename)
 
 
 def delete_dir(dirname):
@@ -109,3 +110,27 @@ def delete_contents(dirname, exclude=None):
       delete_dir(entry)
     else:
       delete_file(entry)
+
+
+# TODO(sbc): Replace with functools.cache, once we update to python 3.7
+def memoize(func):
+  results = {}
+
+  @wraps(func)
+  def helper(*args, **kwargs):
+    assert not kwargs
+    key = (func.__name__, args)
+    if key not in results:
+      results[key] = func(*args)
+    return results[key]
+
+  return helper
+
+
+# TODO: Move this back to shared.py once importing that file becoming side effect free (i.e. it no longer requires a config).
+def set_version_globals():
+  global EMSCRIPTEN_VERSION, EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY
+  filename = path_from_root('emscripten-version.txt')
+  EMSCRIPTEN_VERSION = read_file(filename).strip().strip('"')
+  parts = [int(x) for x in EMSCRIPTEN_VERSION.split('-')[0].split('.')]
+  EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY = parts
